#!/usr/bin/env python3

from ctypes import *

import numpy as np

import os
import sys

try:
    shared_library = None
    shared_library = os.environ['LIB_DARKNET']
    if not os.path.exists(shared_library):
            raise ValueError('Path "{shared_library}" does not exist.')
    else:
        import fnmatch
        if not fnmatch.fnmatch(shared_library, '*.so'):
            raise ValueError(f'{shared_library} is not a shared_library')
except KeyError as exception:
    sys.exit('LIB_DARKNET variable is not set.')
except ValueError as exception:
    sys.exit(exception)

class BOX(Structure):
    _fields_ = [('x', c_float),
                ('y', c_float),
                ('w', c_float),
                ('h', c_float)]


class DETECTION(Structure):
    _fields_ = [('bbox', BOX),
                ('classes', c_int),
                ('prob', POINTER(c_float)),
                ('mask', POINTER(c_float)),
                ('objectness', c_float),
                ('sort_class', c_int)]


class IMAGE(Structure):
    _fields_ = [('w', c_int),
                ('h', c_int),
                ('c', c_int),
                ('data', POINTER(c_float))]


class METADATA(Structure):
    _fields_ = [('classes', c_int),
                ('names', POINTER(c_char_p))]

lib = CDLL(shared_library, RTLD_GLOBAL)
lib.network_width.argtypes = [c_void_p]
lib.network_width.restype = c_int
lib.network_height.argtypes = [c_void_p]
lib.network_height.restype = c_int

predict = lib.network_predict
predict.argtypes = [c_void_p, POINTER(c_float)]
predict.restype = POINTER(c_float)

set_gpu = lib.cuda_set_device
set_gpu.argtypes = [c_int]

make_image = lib.make_image
make_image.argtypes = [c_int, c_int, c_int]
make_image.restype = IMAGE

get_network_boxes = lib.get_network_boxes
get_network_boxes.argtypes = \
    [c_void_p, c_int, c_int, c_float, c_float, POINTER(
      c_int), c_int, POINTER(c_int), c_int]
get_network_boxes.restype = POINTER(DETECTION)

make_network_boxes = lib.make_network_boxes
make_network_boxes.argtypes = [c_void_p]
make_network_boxes.restype = POINTER(DETECTION)

free_detections = lib.free_detections
free_detections.argtypes = [POINTER(DETECTION), c_int]

free_ptrs = lib.free_ptrs
free_ptrs.argtypes = [POINTER(c_void_p), c_int]

network_predict = lib.network_predict
network_predict.argtypes = [c_void_p, POINTER(c_float)]

reset_rnn = lib.reset_rnn
reset_rnn.argtypes = [c_void_p]

load_net = lib.load_network
load_net.argtypes = [c_char_p, c_char_p, c_int]
load_net.restype = c_void_p

load_net_custom = lib.load_network_custom
load_net_custom.argtypes = [c_char_p, c_char_p, c_int, c_int]
load_net_custom.restype = c_void_p

do_nms_obj = lib.do_nms_obj
do_nms_obj.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

do_nms_sort = lib.do_nms_sort
do_nms_sort.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

free_image = lib.free_image
free_image.argtypes = [IMAGE]

letterbox_image = lib.letterbox_image
letterbox_image.argtypes = [IMAGE, c_int, c_int]
letterbox_image.restype = IMAGE

load_meta = lib.get_metadata
lib.get_metadata.argtypes = [c_char_p]
lib.get_metadata.restype = METADATA

load_image = lib.load_image_color
load_image.argtypes = [c_char_p, c_int, c_int]
load_image.restype = IMAGE

rgbgr_image = lib.rgbgr_image
rgbgr_image.argtypes = [IMAGE]

predict_image = lib.network_predict_image
predict_image.argtypes = [c_void_p, IMAGE]
predict_image.restype = POINTER(c_float)

def array_to_image(arr):
    arr = arr.transpose(2, 0, 1)
    c = arr.shape[0]
    h = arr.shape[1]
    w = arr.shape[2]
    arr = np.ascontiguousarray(arr.flat, dtype=np.float32) / 255.0
    data = arr.ctypes.data_as(POINTER(c_float))
    im = IMAGE(w, h, c, data)
    return im, arr



#!/usr/bin/env python3

import cv2
from .cyolo import *
import numpy as np

class BBox(np.ndarray):
    def __new__(cls, x, y, w, h, prob, name):
        cls.name = name
        cls.prob = prob
        obj = np.asarray([x, y, w, h]).view(cls)
        obj.x, obj.y, obj.w, obj.h = obj.view()
        return obj

    def __str__(self):
        x, y, w, h = self.view()
        string = f'x: {x}, y: {y}, w: {w}, h: {h}, '
        string += f'probability: {self.prob}, name: {self.name}'
        return string

    def to_xyxy(self):
        x, y, w, h = self.view()
        return np.array([x, y, x + w, y + h])

    def __array_finalize__(self, obj):
        if obj is None: return


class YOLO(object):
    def __init__(self, config_path,
                 weights_path,
                 data_path,
                 detection_threshold = 0.5,
                 hier_threshold = 0.5,
                 nms_threshold = 0.5):

        if not os.path.exists(config_path):
            raise ValueError(f'Invalid config path: {os.path.abspath(config_path)}')
        if not os.path.exists(weights_path):
            raise ValueError(f'Invalid weight path: {os.path.abspath(weights_path)}')
        if not os.path.exists(data_path):
            raise ValueError(f'Invalid data file path: {os.path.abspath(data_path)}')

        self.net_main = load_net_custom(config_path.encode("ascii"),
                                        weights_path.encode("ascii"),
                                        0, 1)
        self.meta_main = load_meta(data_path.encode("ascii"))

        self.height = lib.network_height(self.net_main)
        self.width = lib.network_width(self.net_main)

        with open(data_path) as metaFH:
            meta_contents = metaFH.read()
            import re
            match = re.search("names *= *(.*)$",
                              meta_contents,
                              re.IGNORECASE | re.MULTILINE)
            if match:
                result = match.group(1)
            else:
                result = None
            if os.path.exists(result):
                with open(result) as namesFH:
                    names_list = namesFH.read().strip().split("\n")
                    self.alt_names = [x.strip() for x in names_list]

        self.threshold = detection_threshold
        self.hier_threshold = hier_threshold
        self.nms = nms_threshold


    def detect(self, image, rgb=False):
        original_h, original_w, _ = image.shape
        image = cv2.resize(image,
                           (self.width, self.height),
                           interpolation=cv2.INTER_CUBIC)[:,:,::-1]
        if not rgb:
            image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        im, arr = array_to_image(image)

        num = c_int(0)
        pnum = pointer(num)

        predict_image(self.net_main, im)
        dets = get_network_boxes(
            self.net_main, im.w, im.h,
            self.threshold,
            self.hier_threshold,
            None, 0, pnum, 0)
        num = pnum[0]

        if self.nms:
            do_nms_sort(dets, num, self.meta_main.classes, self.nms)

        res = []
        for j in range(num):
            for i in range(self.meta_main.classes):
                if dets[j].prob[i] > 0:
                    b = dets[j].bbox

                    # coordinates as percentage
                    x = (b.x-b.w/2)/self.width
                    y = (b.y-b.h/2)/self.height
                    w = b.w/self.width
                    h = b.h/self.height

                    # scale detections to input image
                    x = int(round(x*original_w))
                    y = int(round(y*original_h))
                    w = int(round(w*original_w))
                    h = int(round(h*original_h))

                    bbox = BBox(x, y, w, h, dets[j].prob[i], self.alt_names[i])

                    res.append(bbox)

        free_detections(dets, num)
        return res

    



from setuptools import setup

import os

directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pyyolo',
    version='0.1.2',
    description='Python Darknet YOLO Wrapper',
    author = 'Göktuğ Karakaşlı',
    author_email='karakasligk@gmail.com',
    license='MIT',
    url='https://github.com/goktug97/PyYOLO',
    download_url='https://github.com/goktug97/PyYOLO/archive/v0.1.tar.gz',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=['pyyolo'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    install_requires=[
        'numpy',
    ],
    python_requires='>=3.6',
    include_package_data=True)
